/*******************************************************************************
 * cell.hpp
 * class representing the cell, cellarray and cellcollection objects used during 
 * development
 *
 * Copyright (C) 2006 Joacim Thomassen
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version. 
 * (See accompanying file LICENSE.txt or copy at 
 * http://www.gnu.org/copyleft/gpl.html)
 */

#ifndef CELL_HPP 
#define CELL_HPP 

#include <vector>
#include <string>
class Rule;

/**
 * Cell interface
 */
class Cell
{
   private:
      int m_type;
      
   public:
      Cell();
      Cell(int type);
      ~Cell();
      int GetType();
      void SetType(int type);
      std::string GetTypeString();

      /* Static class memeber constants */
      static const int GROW_NORTH;
      static const int GROW_SOUTH;
      static const int GROW_EAST;
      static const int GROW_WEST;
      static const int TYPE_BLACK;
      static const int TYPE_WHITE;
      static const int DONT_CARE;
      static const int EMPTY;

      static const int TYPE_RED;
      static const int TYPE_BLUE;
      static const int TYPE_GREEN;
      
      static const int NUMBER_OF_TYPES;
};

class CellCollection
{
   private:
     /* not allowed */
     CellCollection();

     Cell* m_c; 
     Cell* m_n; 
     Cell* m_s; 
     Cell* m_e; 
     Cell* m_w; 
     std::vector<Rule*> m_rules;

   public:
     CellCollection(Cell* center, 
	            Cell* north, 
		    Cell* south, 
		    Cell* east, 
		    Cell* west);

     ~CellCollection();
     void AddRule(Rule* rule);

     int Evaluate(int printMatch);
     int GetCellCollectionCenter();
     bool CellCollectionRuleMatch();
     int GetMatchedRuleNumber();
   
};

typedef std::vector<Cell*> CellRow;
typedef std::vector<CellRow*> CellGrid;

class CellArray 
{
 private:
   CellGrid* m_array;
   int m_rows;
   int m_cols;
   CellArray() {}
 public:
   CellArray(int rows, int cols);
   ~CellArray();
   int GetRows() {return m_rows;}
   int GetCols() {return m_cols;}
   Cell* Get(int row, int col);
   int GetType(int row, int col);
   void SetType(int row, int col, int type);
   CellCollection* CreateCellCollection(int row, int col);
   std::string ToString();
};

#endif
