/******************************************************************************
 * exploration.cpp
 * controller class for each exploration/run
 *
 * Copyright (C) 2006 Joacim Thomassen
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version. 
 * (See accompanying file LICENSE.txt or copy at 
 * http://www.gnu.org/copyleft/gpl.html)
 */

#include "exploration.hpp"
#include "tools.hpp"
#include <ctime>
#include <cstdlib>
#include <vector>
#include "config.hpp" 
#include <iostream>
#include <fstream>
#include <string>

#include "boost/date_time/gregorian/gregorian.hpp"

using namespace std;
using namespace boost::posix_time;
using namespace boost::gregorian;

/* default constructor */
Exploration::Exploration( const std::string& name, 
                          const int log_level,
			  const int output,
                          const Config& config,
			  const ptime& timeStarted)
   : m_name(name),
     m_log_level(log_level),
     m_output(output),
     m_config(config),
     m_timeStarted(timeStarted)
{}

void Exploration::Start()
{
   /* initialize random generator */
   srand(time(0));

   // log events to name_date_time.exp - redirect every cout to file
   // open file (with buffer) for writing
   vector<string> filname_units;
   string filename;
   filname_units.push_back(m_name);
   filname_units.push_back(to_iso_string(m_timeStarted.date()));
   filname_units.push_back(to_iso_string(m_timeStarted.time_of_day()));
   filname_units.push_back(string(".exp"));
   Tools::GetInstance()->join(filname_units, '_', filename);
   std::ofstream file(filename.c_str());
   // save output buffer of the passed stream
   std::streambuf* strm_buffer = std::cout.rdbuf();
   // redirect output to the file
   std::cout.rdbuf(file.rdbuf());

   m_config.printConfigSetup();

   /* create my initial generation: */
   Population population( m_config.population_size(), 
			  m_config.genome_size(),
			  m_config.phenotype_size(),
			  m_config.development_steps(),
			  m_config.species(),
			  m_config.seed_population() );
   
   /* run evolution for x generations */
   for (int i = 0; i < m_config.generations(); i++)
   {
      std::cout << "---------------------------------------------" << std::endl;
      std::cout << "Generation: "<< i << std::endl;
      std::cout << "---------------------------------------------" << std::endl;
      population.Evolve();
   }
   if (m_output == 5)
   {
      population.printDevelopment();
   }

   // restore old output buffer of the passed stream
   std::cout.rdbuf(strm_buffer);
}

