/*******************************************************************************
 * exploration.hpp
 * controller class for each exploration/run
 *
 * Copyright (C) 2006 Joacim Thomassen
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version. 
 * (See accompanying file LICENSE.txt or copy at 
 * http://www.gnu.org/copyleft/gpl.html)
 */

#ifndef EXPLORATION_HPP
#define EXPLORATION_HPP

#include <iostream>
#include <string>
#include <cstdlib>
#include <sstream>
#include <vector>

#include "individual.hpp"
#include "simpleindividual.hpp"
#include "neutralindividual.hpp"
#include "flagindividual.hpp"
#include "population.hpp"
#include "config.hpp"

#include "boost/date_time/posix_time/posix_time.hpp"

using namespace boost::posix_time;

class Exploration
{
   private:
      Exploration();
      // name of this exploration defined in configfile
      const std::string m_name; 
      const int m_log_level;
      const int m_output;
      // reference to the config object
      const Config& m_config; 
      const ptime m_timeStarted;
   public:
      Exploration( const std::string& name, 
	           const int log_level,
		   const int output,
	           const Config& config,
		   const ptime& timeStarted );
      void Start();
      ~Exploration() {}
};



#endif
