/**********************************************************************
 * individual.hpp
 * abstract interface class for individuals 
 *
 * Copyright (C) 2006 Joacim Thomassen
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version. 
 * (See accompanying file LICENSE.txt or copy at 
 * http://www.gnu.org/copyleft/gpl.html)
 */
#ifndef INDIVIDUAL_HPP
#define INDIVIDUAL_HPP

#include <vector>
#include <string>
class Cell;
class Rule;
class CellArray;
class SimpleIndividual;
class NeutralIndividual;
class FlagIndividual;
   
/* abstract class Individual */
class Individual
{
   protected:
      std::vector<int> m_genome;
      std::vector<Rule*> m_rules;
      int m_genome_size;
      int m_rows; 
      int m_cols; 
      int m_development_steps;
      bool m_seeded;
      double m_fitness; 
      double m_phenotype_fitness;
      CellArray* m_board;
      Individual(int genome_size,
	         int phenotype_size, 
	         int development_steps,
		 bool seed_population); 
      Individual(Individual* i); 
      virtual Individual* NextIndividual(Individual* i) = 0; 
   public:
      virtual ~Individual() {}
      // initial functions:
      virtual void GenomeFactory() = 0;
      virtual void RuleFactory() = 0;
      // process functions:
      virtual void Evaluate() = 0;
      virtual void Develop() = 0;
      // genetic operators:
      virtual void Mutate() = 0;
      virtual Individual* Clone() = 0;
      // common logging functions:
      void PrintRuleset(); 
      void PrintGenome();
      std::string PrintPhenotype();
      std::string ToString();
      void ToString_Development();
      // common Get functions:
      double GetFitness() { return m_fitness; }
      double GetPhenotypeFitness() { return m_phenotype_fitness; }
      int GetRows() { return m_rows; }
      int GetCols() { return m_cols; }
      std::vector<int> GetGenome() { return m_genome; } 
      int GetGenomeSize() { return m_genome_size; } 
      int GetDevelopment_steps() { return m_development_steps; }
      bool GetSeed_state() { return m_seeded; }
      int GetAction(int ruleNumber);
      int GetCenter(int ruleNumber);
      int GetNorth(int ruleNumber);
      int GetSouth(int ruleNumber);
      int GetEast(int ruleNumber);
      int GetWest(int ruleNumber);
};

#endif

