/*******************************************************************************
 * population.hpp 
 * During the exploration one population object is used to contain all the 
 * individuals. During the evolution AddIndividual is used to populate the 
 * m_individuals vector of the next population for each new generation.
 *
 * Copyright (C) 2006 Joacim Thomassen
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version. 
 * (See accompanying file LICENSE.txt or copy at 
 * http://www.gnu.org/copyleft/gpl.html)
 */

#ifndef POPULATION_HPP
#define POPULATION_HPP

#include <vector>
#include "individual.hpp"
#include "simpleindividual.hpp"
#include "neutralindividual.hpp"
#include "flagindividual.hpp"


class Population
{
   private:
      std::vector<Individual* > m_individuals;
      int m_population_size;
      Individual* fittest;
      Population() {}
   public:
      ~Population();
      Population( int population_size, 
	          int genome_size, 
		  int phenotype_size, 
		  int development_steps, 
		  std::string species,
		  bool seed_population );
      void AddIndividual(Individual* i);
      void Evolve();
      void BreedNextGeneration();
      void Selection();
      void printDevelopment();
      void PrintPopulation();
};

#endif
