/*******************************************************************************
 * rule.cpp
 * class representing the rule objects assosiated with individuals and
 * cellcolletions during development
 *
 * Copyright (C) 2006 Joacim Thomassen
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version. 
 * (See accompanying file LICENSE.txt or copy at 
 * http://www.gnu.org/copyleft/gpl.html)
 */

#include <iostream>
#include "cell.hpp"
#include "rule.hpp"

#define RULE_DEBUG(x)
//#define RULE_DEBUG(x) x
Rule::Rule(int action, 
	   int center, 
	   int north, 
	   int south, 
	   int east, 
	   int west): 
   m_action(action),
   m_c(center),
   m_n(north),
   m_s(south),
   m_e(east),
   m_w(west),
   legality(-1)
{
   RULE_DEBUG(std::cout << "Creating rule a:" << action
                        << " c:" << center 
                        << " n:" << north 
                        << " s:" << south 
                        << " e:" << east 
                        << " w:" << west << std::endl);
  
}

Rule::~Rule() 
{
}

/* Rule Fitness feedback */
/* FIXME: Rulefitness is used narrowly. The compound fitness given for well
 * assembled rules should ideally start from any rulevalue. Only the number of
 * correctly assembled rulevalues in a rule should make the increased fitness
 * score difference, regardless of the rulevalus' placement in the rule. This
 * implementation of rulefitness is used for now as it works (but probably needs
 * more generations to produce an individual with a ruleset to allow
 * development), and gives a simpler implementation. Optimizing is not a goal
 * yet. */
double Rule::RuleFitness()
{
   double ruleScore = 0;
   if (m_action != (Cell::DONT_CARE && Cell::EMPTY))
   {
      ruleScore += 2.0;
   }
   if (m_c != Cell::GROW_NORTH &&
       m_c != Cell::GROW_SOUTH &&
       m_c != Cell::GROW_EAST  &&
       m_c != Cell::GROW_WEST    )
   {
      ruleScore += 2.0;
   }
   if (m_n != Cell::GROW_NORTH &&
       m_n != Cell::GROW_SOUTH &&
       m_n != Cell::GROW_EAST  &&
       m_n != Cell::GROW_WEST    )
   {
      ruleScore += 2.0;
   }
   if (m_s != Cell::GROW_NORTH &&
       m_s != Cell::GROW_SOUTH &&
       m_s != Cell::GROW_EAST  &&
       m_s != Cell::GROW_WEST    )
   {
      ruleScore += 2.0;
   }
   if (m_e != Cell::GROW_NORTH &&
       m_e != Cell::GROW_SOUTH &&
       m_e != Cell::GROW_EAST  &&
       m_e != Cell::GROW_WEST    )
   {
      ruleScore += 2.0;
   }
   if (m_w != Cell::GROW_NORTH &&
       m_w != Cell::GROW_SOUTH &&
       m_w != Cell::GROW_EAST  &&
       m_w != Cell::GROW_WEST    )
   {
      ruleScore += 2.0;
   }
   // favorise Growth combos over Change combos
   // because of lesser probability(solution space)
   if (m_action == Cell::GROW_NORTH ||
       m_action == Cell::GROW_SOUTH ||
       m_action == Cell::GROW_EAST  ||
       m_action == Cell::GROW_WEST    )
   {
      if (m_c == Cell::EMPTY)
      {
	 ruleScore += 19.0; //FIXME: WAS 3.0, depends on NUMBER_OF_CELLTYPES 
	 if (m_n != Cell::GROW_NORTH &&
	     m_n != Cell::GROW_SOUTH &&
	     m_n != Cell::GROW_EAST  &&
	     m_n != Cell::GROW_WEST    )
	 {
	    ruleScore += 25.0;//FIXME: WAS 4.0, depends on NUMBER_OF_CELLTYPES
	    if (m_s != Cell::GROW_NORTH &&
		m_s != Cell::GROW_SOUTH &&
		m_s != Cell::GROW_EAST  &&
		m_s != Cell::GROW_WEST    )
	    {
	       ruleScore += 31.0;//FIXME: WAS 5.0, depends on NUMBER_OF_C.
	       if (m_e != Cell::GROW_NORTH &&
		   m_e != Cell::GROW_SOUTH &&
		   m_e != Cell::GROW_EAST  &&
		   m_e != Cell::GROW_WEST    )
	       {
		  ruleScore += 37.0;//FIXME: WAS 6.0, depends on NUMBER_OF_C.
		  if (m_w != Cell::GROW_NORTH &&
		      m_w != Cell::GROW_SOUTH &&
		      m_w != Cell::GROW_EAST  &&
		      m_w != Cell::GROW_WEST    )
		  {
		     ruleScore += 43.0;//FIXME: WAS 7.0, depends on NUMBER_OF_C.
		  }
	       }
	    }
	 }
      }
   }
   if (m_action != Cell::GROW_NORTH &&
       m_action != Cell::GROW_SOUTH &&
       m_action != Cell::GROW_EAST  &&
       m_action != Cell::GROW_WEST  && 
       m_action != Cell::EMPTY      && 
       m_action != Cell::DONT_CARE    )
   {
      if (m_c != Cell::EMPTY)
      {
	 ruleScore += 3.0;
	 if (m_n != Cell::GROW_NORTH &&
	     m_n != Cell::GROW_SOUTH &&
	     m_n != Cell::GROW_EAST  &&
	     m_n != Cell::GROW_WEST    )
	 {
	    ruleScore += 4.0;
	    if (m_s != Cell::GROW_NORTH &&
		m_s != Cell::GROW_SOUTH &&
		m_s != Cell::GROW_EAST  &&
		m_s != Cell::GROW_WEST    )
	    {
	       ruleScore += 5.0;
	       if (m_e != Cell::GROW_NORTH &&
		   m_e != Cell::GROW_SOUTH &&
		   m_e != Cell::GROW_EAST  &&
		   m_e != Cell::GROW_WEST    )
	       {
		  ruleScore += 6.0;
		  if (m_w != Cell::GROW_NORTH &&
		      m_w != Cell::GROW_SOUTH &&
		      m_w != Cell::GROW_EAST  &&
		      m_w != Cell::GROW_WEST    )
		  {
		     ruleScore += 7.0;
		  }
	       }
	    }
	 }
      }
   }
   return ruleScore;
}

/* Rule legality checks */
bool Rule::IsLegal() 
{
   legality = 1;
   bool isLegal = true;
   // check if legality check already has been performed
   if (legality != -1) // none validated rule
   {
      // growth on non-empty cell
      if (m_action == Cell::GROW_NORTH ||
	  m_action == Cell::GROW_SOUTH ||
	  m_action == Cell::GROW_EAST  ||
	  m_action == Cell::GROW_WEST    )
      {
	 if (m_c != Cell::EMPTY)
	 {
	    legality = 0;
	 }
      }
      // change on empty cell (only growth on empty cells) or DC
      if (m_action != Cell::GROW_NORTH &&
	  m_action != Cell::GROW_SOUTH &&
	  m_action != Cell::GROW_EAST  &&
	  m_action != Cell::GROW_WEST    )
      {
	 if (m_c == Cell::EMPTY)
	 {
	    legality = 0;
	 }
	 if (m_c == Cell::DONT_CARE)
	 {
	    legality = 0;
	 }
      }
      // ALLOW SELF DESTRUCT FOR NOW, NO NEGATIVE IMPACT WITH TODAY'S FITNESS 
      /* self destruct, NOT allowed. might slow down progress?
      if (m_action == Cell::EMPTY) 
      {
	 legality = 0;
      }*/
      // action part is DC, no meaning
      if (m_action == Cell::DONT_CARE)
      {
	 legality = 0;
      }
      // cells of type growth in nabouring cells, do not exist
      if (m_c == Cell::GROW_NORTH ||
	  m_c == Cell::GROW_SOUTH ||
	  m_c == Cell::GROW_EAST  ||
	  m_c == Cell::GROW_WEST  ||
	  m_n == Cell::GROW_NORTH ||
	  m_n == Cell::GROW_SOUTH ||
	  m_n == Cell::GROW_EAST  ||
	  m_n == Cell::GROW_WEST  ||
	  m_s == Cell::GROW_NORTH ||
	  m_s == Cell::GROW_SOUTH ||
	  m_s == Cell::GROW_EAST  ||
	  m_s == Cell::GROW_WEST  ||
	  m_e == Cell::GROW_NORTH ||
	  m_e == Cell::GROW_SOUTH ||
	  m_e == Cell::GROW_EAST  ||
	  m_e == Cell::GROW_WEST  ||
	  m_w == Cell::GROW_NORTH ||
	  m_w == Cell::GROW_SOUTH ||
	  m_w == Cell::GROW_EAST  ||
	  m_w == Cell::GROW_WEST    )
      {
	 legality = 0;
      }
   }
   switch(legality)
   {
      case 0: 
	 isLegal = false;
	 break;
      case 1: 
	 isLegal = true;
	 break;
      default:
	 isLegal = false;
   }
   return isLegal;
}

bool Rule::Match(Cell* c, Cell* n, Cell* s, Cell* e, Cell* w)
{
   if (legality == 1)
   {
      // check if all conditions are met
      return ( (c->GetType() == m_c || m_c == Cell::DONT_CARE ) &&
	       (n->GetType() == m_n || m_n == Cell::DONT_CARE ) && 
	       (s->GetType() == m_s || m_s == Cell::DONT_CARE ) &&
	       (e->GetType() == m_e || m_e == Cell::DONT_CARE ) &&
	       (w->GetType() == m_w || m_w == Cell::DONT_CARE ) );
   }
   else
   {
      return false;
   }
}


int Rule::GetAction() 
{
   return m_action;
}

void Rule::ToString()
{
   std::cout << m_action << " " << m_c << " " << m_n << " " << m_s << " " << m_e 
             << " " << m_w << " " << std::endl;
}
