/*******************************************************************************
 * tools.cpp
 * tools collection class with common functions and variables 
 *
 * Copyright (C) 2006 Joacim Thomassen
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version. 
 * (See accompanying file LICENSE.txt or copy at 
 * http://www.gnu.org/copyleft/gpl.html)
 */

#include <boost/lexical_cast.hpp>
#include <boost/regex.hpp>
#include <iostream>
#include <string>
#include <fstream>
#include <cstdlib>
#include <vector>
#include "tools.hpp"

using namespace std;

// define static members  
Tools* Tools::m_instance = 0; 

// Not threadsafe
Tools* Tools::GetInstance()
{
   if (m_instance == 0)
   {
      m_instance = new Tools();
   }
   return m_instance;
}

// return m_seed_genome vector
vector<int>& Tools::GetSeed()
{
   return m_seed_genome;
}

// join strings together, used to generate uniqe filnames 
void Tools::join(const vector<string>& v, char c, string& s)
{
   s.clear();
   for (vector<string>::const_iterator p = v.begin(); p != v.end(); ++p)
   {
      s += *p;
      if (p < (v.end() - 2)) // allows extension like .exp
      {
	 s += c;
      }
   }
}

// read seed from file given as argument on program start ( -s [filname] )
void Tools::SetSeed(const char* seed_file)
{
   std::string tmp_seed;
   std::ifstream in(seed_file); // seed file
   // read first line only
   std::getline(in, tmp_seed, '\n');
   boost::regex re("\\s+"); // match one or more spaces
   boost::sregex_token_iterator p(tmp_seed.begin(), tmp_seed.end(), re, -1);
   boost::sregex_token_iterator end;
   while (p != end)
   {
      m_seed_genome.push_back(boost::lexical_cast<int>(*p++));
   }
   std::cout << "Seed accepted: ";
   std::vector<int>::iterator seedit;
   for (seedit = m_seed_genome.begin();seedit != m_seed_genome.end();++seedit)
   {
      std::cout << (*seedit) << " ";
   }
   std::cout << std::endl;
}
