/*******************************************************************************
 * ruleviz.c 
 * makes a graphical ruleset (eps) from a textfile represented ruleset
 *
 * This is a modified version of Kjetil Aamodt's sblock.c
 *
 * Copyright (C) 2005 Joacim Thomassen 
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version. 
 * (See accompanying file LICENSE.txt or copy at 
 * http://www.gnu.org/copyleft/gpl.html)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int rulenumber;
int rulesize;
int a[32][32];
char b[32][32];

char outfilename[30];
int COLOR = 1;
char *color[] = {"1.0 1.0 0.0", //rgb color for type 0
		 "1.0 1.0 1.0", //type 1
		 "1.0 1.0 1.0", //type 2
		 "1.0 1.0 1.0",  //type 3
		 "1.0 1.0 1.0", //type 4
		 "0.0 0.0 0.0", //type 5
		 "1.0 1.0 1.0", //type 6
		 "1.0 1.0 1.0", //type 7
		 "0.5 0.5 0.5", //type 8
		 "1.0 0.0 0.0", //type 9
		 "0.0 0.0 1.0", //type 10
		 "0.0 1.0 0.0", //type 11
		 "1.0 1.0 1.0" //type 12
		 };
char *fontcolor[] = {"0.0 0.0 0.0", //rgb color for type 0
		    "0.0 0.0 0.0", //type 1
		    "0.0 0.0 0.0", //type 2
		    "0.0 0.0 0.0",  //type 3
		    "0.0 0.0 0.0", //type 4
		    "0.0 0.0 0.0", //type 5
		    "0.0 0.0 0.0", //type 6
		    "0.0 0.0 0.0", //type 7
		    "0.5 0.5 0.5", //type 8
		    "1.0 0.0 0.0", //type 9
		    "0.0 0.0 1.0", //type 10
		    "0.0 1.0 0.0", //type 11
		    "0.0 0.0 0.0" //type 12
		 };
char *gene[] = {"?", // name of type 0
                "Gn", // type 1
                "Gs", // type 2
                "Ge", // type 3
                "Gw", // type 4
                "o", // type 5
                "o", // type 6
                "DC", // type 7
                "o", // type 8
                "o", // type 9
                "o", // type 10
                "o", // type 11
                "?", // type 12
                };
char *rule[] = {"#", // rule-id 
                "R", // result[action]
                "C", // Center[cond.]
                "N", // North[cond.]
                "S", // South[cond.}
                "E", // East[cond.]
                "W", // West[cond.]
		};
char *gray[] = {"1.0",
		"0.3",
		"0.8",
		"0.0"};
int r = 2; //radius
int d = 5; //diameter

void writeTypePS(char* filename);
void writeStatePS(char* filename);
void readFile(char *filename);
void itoa(int a, char *s);
void createSample(int type);
void createFilename(char filename[], char postfix[], int n);

int main(int argc, char* argv[]){
  int k;
  for(k = 0; k < 12; k++){
    createSample(k);
  }

  rulenumber = 16; 
  rulesize = 7; 
  if(argc == 4){
    rulenumber = atoi(argv[2]);
    rulesize = atoi(argv[3]);
  }
  readFile(argv[1]);
  return 0;
}

void readFile(char *filename){
  FILE* in;
  int i,j;
  char line[40];
  int typescounter = 0;

  in = fopen(filename, "r");
  while(fgets(line, 40, in) != 0){
    if(0 == strcmp(line, "My ruleset: \n")){
      printf("Ruleset found!\n");
      printf("My ruleset: %i\n", typescounter);
      for(i=0; i < rulenumber; i++){
	for(j=0; j < rulesize-1; j++){
	  fscanf(in, "%i", &a[i][j]);
	  printf("%i ", a[i][j]);
	}
	printf("\n"); fflush(stdout);
      }
      createFilename(outfilename, "types_", typescounter);
      printf("%s\n\n", outfilename); fflush(stdout);
      writeTypePS(outfilename);
      typescounter++;
    }
  }
  fclose(in);
}

/* Print sample image of type */
void createSample(int type){
  rulenumber = rulesize = 1;
  a[0][0] = type;
  createFilename(outfilename, "sample_", type);
  writeTypePS(outfilename);
}


/* Write matrix states to ps-file
 */
void writeTypePS(char* filename)
{
  FILE *out;
  int i,j;

  if ((out = fopen(filename, "wb")) == NULL)
  {
    printf("Can not write to file %s\n", filename);
  }
  else
  {
    fprintf(out, "%%%%!PS-Adobe-3.0 EPSF-3,0\n");
    fprintf(out, "%%%%BoundingBox: ");
    fprintf(out, "-2 -2 %i %i\n", rulesize*2*d-1, (rulenumber+1)*d+2);
    fprintf(out, "/en {1 mul} def\n");

    // rule-id in first row
    for (j = 0; j < rulesize; j++)
    {
       fprintf(out, "/Helvetica findfont\n");
       fprintf(out, "5 scalefont setfont\n");
       fprintf(out, "%i %i moveto\n", (j*2*d), ((rulenumber+1)*d - (d-1)));
       fprintf(out, "%s setrgbcolor", fontcolor[0]);
       fprintf(out, "(%2s) show\n", rule[j]); // write rule-id
       fprintf(out, "gsave\n");
    }
    // horizontal top frame line
    fprintf(out, "0 setgray\n");
    fprintf(out, "0.1 setlinewidth\n");
    fprintf(out, "%i %i moveto\n", -1, (rulenumber+1)*d + 1);
    fprintf(out, "%i %i lineto\n", 2*d*rulesize-1, (rulenumber+1)*d + 1);
    fprintf(out, "closepath\n");
    fprintf(out, "gsave\n");
    // horizontal  middle frame line
    fprintf(out, "0 setgray\n");
    fprintf(out, "0.1 setlinewidth\n");
    fprintf(out, "%i %i moveto\n", 0, (rulenumber+1)*d - (d));
    fprintf(out, "%i %i lineto\n", 2*d*rulesize-1, (rulenumber+1)*d - (d));
    fprintf(out, "closepath\n");
    fprintf(out, "gsave\n");
    // horizontal bottom frame line
    fprintf(out, "0 setgray\n");
    fprintf(out, "0.1 setlinewidth\n");
    fprintf(out, "%i %i moveto\n", -1, -2);
    fprintf(out, "%i %i lineto\n", 2*d*rulesize-1, -2);
    fprintf(out, "closepath\n");
    fprintf(out, "gsave\n");
    // vertical frame lines
    for (i = 0; i < (rulesize+1); i++)
    {
       fprintf(out, "0 setgray\n");
       fprintf(out, "0.1 setlinewidth\n");
       fprintf(out, "%i %i moveto\n", i*2*d-(d/4), (rulenumber*d) + d + 1);
       fprintf(out, "%i %i lineto\n", i*2*d-(d/4), -2);
       fprintf(out, "closepath\n");
       fprintf(out, "gsave\n");
    }

    for (i = 0; i < rulenumber ; i++)
    {
      for (j = 0; j < rulesize; j++)
      {
	   if (j == 0)
	   {
	      fprintf(out, "/Helvetica findfont\n");
	      fprintf(out, "5 scalefont setfont\n");
	      fprintf(out, "%i %i moveto\n", (j*2*d), ((rulenumber+1)*d - d) - d*(i+1));
	      fprintf(out, "%s setrgbcolor", fontcolor[0]);
	      fprintf(out, "(%2i) show\n", i); // write rule-id
	      fprintf(out, "gsave\n");
	   }
	   else
	   {
	      if ( a[i][j-1] == 5 || a[i][j-1] == 6 || a[i][j-1] == 8 || 
		   a[i][j-1] == 9 || a[i][j-1] == 10 || a[i][j-1] == 11 )
	      {
		 fprintf(out, "stroke\n");
		 fprintf(out, "0 setgray\n");
		 fprintf(out, "0.1 setlinewidth\n");
		 fprintf(out, "%i %i %i 0 360 arc\n", (j*2*d + (d-r)), (rulenumber+1)*d - (d-r) - d*(i+1), r);
		 fprintf(out, "closepath\n");
		 fprintf(out, "gsave\n");
		 fprintf(out, "%s setrgbcolor", color[a[i][j-1]]);
		 fprintf(out, " fill\n");
		 fprintf(out, "grestore\n");
		 fprintf(out, "stroke\n");
		 fprintf(out, "gsave\n");
	      }
	      else
	      {
		 fprintf(out, "/Helvetica findfont\n");
		 fprintf(out, "5 scalefont setfont\n");
		 fprintf(out, "%i %i moveto\n", (j*2*d), ((rulenumber+1)*d - d) - d*(i+1));
		 fprintf(out, "%s setrgbcolor", fontcolor[a[i][j-1]]);
		 fprintf(out, "(%2s) show\n", gene[a[i][j-1]]);
		 fprintf(out, "gsave\n");
	      }
	   }
      }
    }
    fclose(out);
  }
}


void createFilename(char filename[], char postfix[], int n){
  char number[4];
  strcpy(filename, "output/");
  strcat(filename, postfix);
  itoa(n, number);
  strcat(filename, number);
  strcat(filename, ".eps");
}

void itoa(int number, char s[]){
  s[3] = '\0';
  s[2] = '0' + (number%10);
  number /= 10;
  s[1] = '0' + (number%10);
  number /= 10;
  s[0] = '0' + (number%10);
}
