/*******************************************************************************
 * sblock.c 
 * makes a graphical cell array (eps) from a textfile represented matrix
 *
 * Copyright (C) 2005 Kjetil Aamodt 
 *
 * Released under the GNU General Public License by Joacim Thomassen in 2006,
 * as per agreement with Kjetil Aamodt.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or any later version. 
 * (See accompanying file LICENSE.txt or copy at 
 * http://www.gnu.org/copyleft/gpl.html)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int n;
int m;
int a[32][32];
char b[32][32];

char outfilename[30];
int COLOR = 1;
char *color[] = {"1.0 1.0 0.0", //rgb color for type 0
		 "0.7 0.7 0.7", //type 1
		 "0.7 0.7 0.0", //type 2
		 "0.0 0.7 0.0",  //type 3
		 "0.0 0.7 0.7", //type 4
		 "0.0 0.0 0.0", //type 5
		 "1.0 1.0 1.0", //type 6
		 "0.7 0.7 0.7", //type 7
		 "0.5 0.5 0.5", //type 8
		 "1.0 0.0 0.0", //type 9
		 "0.0 0.0 1.0", //type 10
		 "0.0 1.0 0.0", //type 11
		 "0.7 1.0 1.0" //type 12
		 };
char *gray[] = {"1.0",
		"0.3",
		"0.8",
		"0.0"};
int r = 5; //radius

void writeTypePS(char* filename);
void writeStatePS(char* filename);
void readFile(char *filename);
void itoa(int a, char *s);
void createSample(int type);
void createFilename(char filename[], char postfix[], int n);

int main(int argc, char* argv[]){
  int k;
  for(k = 0; k < 12; k++){
    createSample(k);
  }

  n = m = 8; //default
  if(argc == 4){
    n = atoi(argv[2]);
    m = atoi(argv[3]);
  }
  readFile(argv[1]);
  return 0;
}

void readFile(char *filename){
  FILE* in;
  int i,j;
  char line[40];
  int statescounter = 0;
  int typescounter = 0;

  in = fopen(filename, "r");
  while(fgets(line, 40, in) != 0){
    if(0 == strcmp(line, "Printing types:\n")){
      printf("Types - matrix: %i\n", typescounter);
      for(i=0; i < n; i++){
	for(j=0; j < m; j++){
	  fscanf(in, "%i", &a[i][j]);
	  printf("%i ", a[i][j]);
	}
	printf("\n"); fflush(stdout);
      }
      createFilename(outfilename, "types_", typescounter);
      printf("%s\n\n", outfilename); fflush(stdout);
      writeTypePS(outfilename);
      typescounter++;
    }
    else if(0 == strcmp(line, "Printing states:\n")){
      printf("States - matrix: %i\n", statescounter);
      for(i=0; i < n; i++){
	for(j=0; j < m; j++){
	  fscanf(in, "%c%*c", &b[i][j]);
	  printf("%c ", b[i][j]);
	}
	while(fgetc(in) != '\n');
	printf("\n"); fflush(stdout);
      }
      createFilename(outfilename, "states_", statescounter);
      printf("%s\n\n", outfilename); fflush(stdout);
      writeStatePS(outfilename);
      statescounter++;
    }
  }
  fclose(in);
}

/* Print sample image of type */
void createSample(int type){
  n = m = 1;
  a[0][0] = type;
  createFilename(outfilename, "sample_", type);
  writeTypePS(outfilename);
}


void writeStatePS(char* filename){
  FILE *out;
  int i,j;
  if ((out = fopen(filename, "wb")) == NULL){
    printf("Can not write to file %s\n", filename);
  }
  else{
    fprintf(out, "%%%%!PS-Adobe-3.0 EPSF-3,0\n");
    fprintf(out, "%%%%BoundingBox: ");
    fprintf(out, "0 0 %i %i\n", n*2*r, m*2*r);
    fprintf(out, "/en {1 mul} def\n");

    for (i = 0; i < n ; i++){
      for (j = 0; j < m; j++){
	fprintf(out, "0 setgray\n");
	fprintf(out, "0.1 setlinewidth\n");
	fprintf(out, "%i %i %i 0 360 arc\n", (j*2*r + r), (m*2*r - r) - 2*r*i, r);
	fprintf(out, "closepath\n");

	if(b[i][j] == 'X'){
	  fprintf(out, "gsave\n");
	  fprintf(out, "fill\n");
	  fprintf(out, "grestore\n");
	}
	fprintf(out, "stroke\n");
      }
    }
    fclose(out);
  }
}

/* Write matrix states to ps-file
 */
void writeTypePS(char* filename){
  FILE *out;
  int i,j;

  if ((out = fopen(filename, "wb")) == NULL){
    printf("Can not write to file %s\n", filename);
  }
  else{
    fprintf(out, "%%%%!PS-Adobe-3.0 EPSF-3,0\n");
    fprintf(out, "%%%%BoundingBox: ");
    fprintf(out, "0 0 %i %i\n", n*2*r, m*2*r);
    fprintf(out, "/en {1 mul} def\n");

    for (i = 0; i < n ; i++){
      for (j = 0; j < m; j++){
	fprintf(out, "0 setgray\n");
	fprintf(out, "0.1 setlinewidth\n");
	fprintf(out, "%i %i %i 0 360 arc\n", (j*2*r + r), (m*2*r - r) - 2*r*i, r);
	fprintf(out, "closepath\n");
	fprintf(out, "gsave\n");

	if(COLOR == 1)
	  fprintf(out, "%s setrgbcolor", color[a[i][j]]);
	else
	  fprintf(out, "%s setgray", a[i][j]);

	fprintf(out, " fill\n");
	fprintf(out, "grestore\n");
	fprintf(out, "stroke\n");
      }
    }
    fclose(out);
  }
}


void createFilename(char filename[], char postfix[], int n){
  char number[4];
  strcpy(filename, "output/");
  strcat(filename, postfix);
  itoa(n, number);
  strcat(filename, number);
  strcat(filename, ".eps");
}

void itoa(int number, char s[]){
  s[3] = '\0';
  s[2] = '0' + (number%10);
  number /= 10;
  s[1] = '0' + (number%10);
  number /= 10;
  s[0] = '0' + (number%10);
}
